@@ -19,12 +19,15 @@ use Data::Dumper;
 
 use Parse::Nibbler;
 
-our $VERSION = '1.05';
+our $VERSION = '1.07';
+
+
 
 
 #############################################################################
 # Lexer
 #############################################################################
+#############################################################################
 sub Lexer
 #############################################################################
 {
@@ -32,59 +35,58 @@ sub Lexer
 
   while(1)
     {
-      my $line = $p->{line_number};
-      my $col = pos($p->{current_line});
+      my $line = $p->[Parse::Nibbler::line_number];
+      my $col = pos($p->[Parse::Nibbler::current_line]);
 
       # if at end of line
-      if( 
-	 ( length($p->{current_line}) == 0 )
-	 or
-	 ( length($p->{current_line}) == pos($p->{current_line}) )
+      if (
+	  length($p->[Parse::Nibbler::current_line]) ==
+	  pos($p->[Parse::Nibbler::current_line])
 	 )
 	{
-	  $p->{line_number} ++;
-	  # print "line ". $p->{line_number}."\n";
-	  my $fh = $p->{handle};
-	  $p->{current_line} = <$fh>;
+	  $p->[Parse::Nibbler::line_number] ++;
+	  # print "line ". $p->[Parse::Nibbler::line_number]."\n";
+	  my $fh = $p->[Parse::Nibbler::handle];
+	  $p->[Parse::Nibbler::current_line] = <$fh>;
 
-	  unless(defined($p->{current_line}))
+	  unless(defined($p->[Parse::Nibbler::current_line]))
 	    {
 	      return bless [ '!EOF!', '!EOF!', $line, $col ], 'Lexical';
 	    }
 
-	  chomp($p->{current_line});
-	  pos($p->{current_line}) = 0;
+	  chomp($p->[Parse::Nibbler::current_line]);
+	  pos($p->[Parse::Nibbler::current_line]) = 0;
 	  redo;
 	}
 
       # delete any leading whitespace and check it again
-      if($p->{current_line} =~ /\G\s+/gc)
+      if($p->[Parse::Nibbler::current_line] =~ /\G\s+/gc)
 	{
 	  redo;
 	}
 
       # look for comment to end of line
-      if($p->{current_line} =~ /\G\/\/.*/gc)
+      if($p->[Parse::Nibbler::current_line] =~ /\G\/\/.*/gc)
 	{
 	  redo;
 	}
 
       # look for identifier (hierarchical)
       my $identifier = '';
-      if( $p->{current_line} =~ /\G\$/gc )
+      if( $p->[Parse::Nibbler::current_line] =~ /\G\$/gc )
 	{
 	  $identifier = "\$";   # system task name
 	}
 
       while(1)
 	{
-	  if ( $p->{current_line} =~ /\G([a-zA-Z_][a-zA-Z0-9_\$]*)/gc )
+	  if ( $p->[Parse::Nibbler::current_line] =~ /\G([a-zA-Z_][a-zA-Z0-9_\$]*)/gc )
 	    {$identifier .= $1;}
 
-	  elsif($p->{current_line} =~ /\G(\\[^\s]+)\s/gc) 
+	  elsif($p->[Parse::Nibbler::current_line] =~ /\G(\\[^\s]+)\s/gc) 
 	    {
 	      $identifier .= $1;
-	      pos($p->{current_line}) = pos($p->{current_line}) - 1; 
+	      pos($p->[Parse::Nibbler::current_line]) = pos($p->[Parse::Nibbler::current_line]) - 1; 
 	    }
 
 	  # no match and no accumulated identifier string
@@ -93,7 +95,7 @@ sub Lexer
 
 	  $p->report_error("bad identifier") if ($identifier =~ /\.$/);
 
-	  if ($p->{current_line} =~ /\G\./gc)
+	  if ($p->[Parse::Nibbler::current_line] =~ /\G\./gc)
 	    {
 	      $identifier .= '.';
 	      redo;
@@ -135,12 +137,12 @@ sub Lexer
       # will ignore optional sign at beginning of number for now.
       # will hope that parser can handle it in rules.
 
-      if ($p->{current_line} =~ /\G([0-9][0-9_]*)/gc)
+      if ($p->[Parse::Nibbler::current_line] =~ /\G([0-9][0-9_]*)/gc)
 	{
 	  $number .= $1;
 	}
 
-      if ( $p->{current_line} =~ /\G(\'[dDoObBhH])/gc )
+      if ( $p->[Parse::Nibbler::current_line] =~ /\G(\'[dDoObBhH])/gc )
 	{
 	  $number .= $1;
 	  $number = lc($number);
@@ -153,7 +155,7 @@ sub Lexer
 
 	  if($number=~/d$/)
 	    {
-	      $p->{current_line} =~ /\G([0-9][0-9_]*)/gc;
+	      $p->[Parse::Nibbler::current_line] =~ /\G([0-9][0-9_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid decimal number") 
@@ -162,7 +164,7 @@ sub Lexer
 	    }
 	  elsif($number=~/o$/)
 	    {
-	      $p->{current_line} =~ /\G([xXzZ0-7][xXzZ0-7_]*)/gc;
+	      $p->[Parse::Nibbler::current_line] =~ /\G([xXzZ0-7][xXzZ0-7_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid octal number") 
@@ -171,7 +173,7 @@ sub Lexer
 	    }
 	  elsif($number=~/b$/)
 	    {
-	      $p->{current_line} =~ /\G([xXzZ01][xXzZ01_]*)/gc;
+	      $p->[Parse::Nibbler::current_line] =~ /\G([xXzZ01][xXzZ01_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid binary number") 
@@ -180,7 +182,7 @@ sub Lexer
 
 	  elsif($number=~/h$/)
 	    {
-	      $p->{current_line} =~ /\G([xXzZ0-9a-fA-F][xXzZ0-9a-fA-F_]*)/gc;
+	      $p->[Parse::Nibbler::current_line] =~ /\G([xXzZ0-9a-fA-F][xXzZ0-9a-fA-F_]*)/gc;
 	      unless(defined($1))
 		{
 		  $p->report_error("Lex error: invalid hexadecimal number") 
@@ -197,27 +199,29 @@ sub Lexer
 	  $number =~ s/\s//g;
 	  return bless ['Number', $number, $line, $col ], 'Lexical';
 	}
-
-      if( $p->{current_line} =~ /\G(\.[0-9][0-9_]*)/gc )
+      else  # not a based number ('h), is it a float with exponent
 	{
-	  $number .= $1;
-	}
+	  if( $p->[Parse::Nibbler::current_line] =~ /\G(\.[0-9][0-9_]*)/gc )
+	    {
+	      $number .= $1;
+	    }
 
-      if( $p->{current_line} =~ /\G(\s*[eE]\s*[+-]*\s*[0-9][0-9_]*)/gc )
-	{
-	  $number .= $1;
-	}
+	  if( $p->[Parse::Nibbler::current_line] =~ /\G(\s*[eE]\s*[+-]*\s*[0-9][0-9_]*)/gc )
+	    {
+	      $number .= $1;
+	    }
 
-      if(length($number)>0)
-	{
-	  $number = lc($number);
-	  $number =~ s/\s//g;
-	  $number =~ s/_//g;
-	  return bless ['Number', $number, $line, $col ], 'Lexical';
-	}
+	  if(length($number)>0)
+	    {
+	      $number = lc($number);
+	      $number =~ s/\s//g;
+	      $number =~ s/_//g;
+	      return bless ['Number', $number, $line, $col ], 'Lexical';
+	    }
+	  }
 
       # else get a single character and return it.
-      $p->{current_line} =~ /\G(.)/gc;
+      $p->[Parse::Nibbler::current_line] =~ /\G(.)/gc;
       return bless [$1, $1, $line, $col ], 'Lexical';
 
     }
@@ -227,6 +231,7 @@ sub Lexer
 
 
 
+
 ###############################################################################
 Register 
 ( 'Number', sub 
@@ -259,7 +264,7 @@ Register
 ( 'SourceText', sub 
 ###############################################################################
   {
-    $_[0]->DescriptionArgs('{*}');
+    $_[0]->Description('{*}');
   }
 );
 
@@ -272,18 +277,28 @@ Register
   }
 );
 
+
+my $module_name;
+
 ###############################################################################
 Register 
 ( 'ModuleDeclaration', sub 
 ###############################################################################
   {
     my $p = $_[0];
+    $module_name=undef;
     $p->ValueIs('module');
+    $module_name = $p->[Parse::Nibbler::list_of_rules_in_progress]->[1];
     $p->TypeIs('Identifier');
-    $p->ListOfPortsArgs('{?}');
+    $p->ListOfPorts  if ($p->PeekValue eq '(');
     $p->ValueIs(';');
-    $p->ModuleItemArgs('{*}');
+    $p->ModuleItem('{*}');
     $p->ValueIs('endmodule');
+  },
+
+  sub
+  {
+    return $module_name;
   }
 );
 
@@ -314,7 +329,7 @@ Register
 ( 'NamedPortExpressionList', sub 
 ###############################################################################
   {
-    $_[0]->NamedPortExpressionArgs('{+}/,/');
+    $_[0]->NamedPortExpression('{+}/,/');
   }
 );
 
@@ -323,7 +338,7 @@ Register
 ( 'AnonPortExpressionList', sub 
 ###############################################################################
   {
-    $_[0]->AnonPortExpressionArgs('{+}/,/');
+    $_[0]->AnonPortExpression('{+}/,/');
   }
 );
 
@@ -333,11 +348,11 @@ Register
 ( 'NamedPortExpression', sub 
 ###############################################################################
   {
-    my $p = shift;
+    my $p = $_[0];
     $p->ValueIs('.');
     $p->TypeIs('Identifier');
     $p->ValueIs('(');
-    $p->AnonPortExpressionArgs('{?}');
+    $p->AnonPortExpression('{?}');
     $p->ValueIs(')');
   }
 );
@@ -348,10 +363,15 @@ Register
 ( 'AnonPortExpression', sub 
 ###############################################################################
   {
-    $_[0]->AlternateRules
-      (
-       'PortReference', 'ConcatenatedPortReference'
-      );
+    my $p = $_[0];
+    if ($p->PeekValue eq '{')
+      {
+	$p->ConcatenatedPortReference;
+      }
+    else
+      {
+	$p->PortReference;
+      }
   }
 );
 
@@ -360,10 +380,15 @@ Register
 ( 'PortReference', sub 
 ###############################################################################
   {
-    $_[0]->AlternateRules
-      (
-       'IdentifierWithPossibleBitSpecifier', 'Number'
-      );
+    my $p = $_[0];
+    if ($p->PeekType eq 'Identifier')
+      {
+	$p->IdentifierWithPossibleBitSpecifier;
+      }
+    else
+      {
+	$p->Number;
+      }
   }
 );
 
@@ -375,7 +400,7 @@ Register
   {
     my $p = $_[0];
     $p->TypeIs('Identifier');
-    $p->BitSpecifierArgs('{?}');
+    $p->BitSpecifier  if ($p->PeekValue eq '[');
   }
 );
 
@@ -389,7 +414,7 @@ Register
     my $p = $_[0];
     $p->ValueIs('[');
     $p->TypeIs('Number');
-    $p->ColonNumberArgs('{?}');
+    $p->ColonNumber  if ($p->PeekValue eq ':');
     $p->ValueIs(']');
   }
 );
@@ -412,7 +437,7 @@ Register
   {
     my $p = $_[0];
     $p->ValueIs('{');
-    $p->PortReferenceArgs('{+}/,/');
+    $p->PortReference('{+}/,/');
     $p->ValueIs('}');
   }
 );
@@ -422,11 +447,15 @@ Register
 ( 'ModuleItem', sub 
 ###############################################################################
   {
-    $_[0]->AlternateRules
-      ( 
-       'DirectionDeclaration',
-       'ModuleInstantiation'
-      );
+    my $p = $_[0];
+    if ($p->PeekValue =~ /input|output|inout/)
+      {
+	$p->DirectionDeclaration;
+      }
+    else
+      {
+       $p->ModuleInstantiation;
+      };
   }
 );
 
@@ -438,8 +467,8 @@ Register
   {
     my $p = $_[0];
     $p->AlternateValues('input', 'output', 'inout');
-    $p->RangeArgs('{?}');
-    $p->PortIdentifierArgs('{+}');
+    $p->Range if ($p->PeekValue eq '[');
+    $p->PortIdentifier('{+}');
     $p->ValueIs(';');
   }
 );
@@ -476,8 +505,8 @@ Register
   {
     my $p = $_[0];
     $p->TypeIs('Identifier');
-    $p->ParameterValueAssignmentArgs('{?}');
-    $p->ModuleInstanceArgs('{+}/,/');
+    $p->ParameterValueAssignment  if ($p->PeekValue eq '#');
+    $p->ModuleInstance('{+}/,/');
     $p->ValueIs(';');
   }
 );
@@ -490,7 +519,7 @@ Register
     my $p = $_[0];
     $p->ValueIs('#');
     $p->ValueIs('(');
-    $p->PortListArgs('{?}');
+    $p->PortList('{?}');
     $p->ValueIs(')');
 
   }
@@ -504,7 +533,7 @@ Register
     my $p = $_[0];
     $p->TypeIs('Identifier');
     $p->ValueIs('(');
-    $p->PortListArgs('{?}');
+    $p->PortList('{?}');
     $p->ValueIs(')');
 
   }
